import typer
from typer import Typer

from sensamag_utility import exporter, language
from sensamag_utility.connection_manager import ConnectionManager

app = Typer(no_args_is_help=True)
connection_manager = ConnectionManager()


@app.callback(invoke_without_command=True)
def connection(
    user: str = None,
    password: str = None,
    host: str = None,
    port: int = None,
    database: str = None,
    reset: bool = False,
):
    if reset:
        connection_manager.reset_connection()
    connection_manager.set_connection(user, password, host, port, database)


@app.command()
def addlang(
    name: str = typer.Option(..., prompt=True),
    culture: str = typer.Option(..., prompt=True),
    priority: int = typer.Option(..., prompt=True),
):
    with connection_manager.get_connection() as conn:
        language.add_language(conn, name, culture, priority)


@app.command()
def removelang(lang_id: int = typer.Option(..., prompt=True)):
    with connection_manager.get_connection() as conn:
        language.remove_language(conn, lang_id)


@app.command()
def listlang():
    with connection_manager.get_connection() as conn:
        language.list_languages(conn)


@app.command()
def exportdb(path: str = typer.Option(..., prompt=True)):
    with connection_manager.get_connection() as conn:
        exporter.export_db(conn, path)



import sys

import mariadb
from rich import print

DEFAULT_USER = "sensamag_utility"
DEFAULT_PASSWORD = "verystrongpassword"
DEFAULT_HOST = "127.0.0.1"
DEFAULT_PORT = 3308
DEFAULT_DATABASE = "sensamag_sp"


class ConnectionManager:
    def __init__(self):
        self.user = DEFAULT_USER
        self.password = DEFAULT_PASSWORD
        self.host = DEFAULT_HOST
        self.port = DEFAULT_PORT
        self.database = DEFAULT_DATABASE

    def reset_connection(self):
        self.user = DEFAULT_USER
        self.password = DEFAULT_PASSWORD
        self.host = DEFAULT_HOST
        self.port = DEFAULT_PORT
        self.database = DEFAULT_DATABASE
        print("> [bold yellow]Connection parameters reset to default!")

    def set_connection(
        self,
        user: str = None,
        password: str = None,
        host: str = None,
        port: int = None,
        database: str = None,
    ):
        if user is not None:
            self.user = user
        if password is not None:
            self.password = password
        if host is not None:
            self.host = host
        if port is not None:
            self.port = port
        if database is not None:
            self.database = database
        print("> [bold yellow]New connection parameters:")
        self.print_params()

    def print_params(self):
        print(
            f"""
        | > User: {self.user}
        | > Password: {self.password}
        | > Host: {self.host}
        | > Port: {self.port}
        | > Database: {self.database}
        """
        )

    def get_connection(self):
        try:
            conn = mariadb.connect(
                user=self.user,
                password=self.password,
                host=self.host,
                port=self.port,
                database=self.database,
            )
            print("> [bold green]Connected successfully!")
            return conn
        except mariadb.Error as exception:
            print(f"> [bold red]Error connecting to MariaDB[/]: {exception}")
            sys.exit(1)



import csv
import os


def read_data(path: str):
    with open(path, newline="", encoding="utf-8") as csvfile:
        reader = csv.DictReader(csvfile)
        data = []
        for row in reader:
            # skip empty lines
            if not row[reader.fieldnames[0]]:
                continue
            data.append(row)
        return data


def write_data(path: str, data, columns):
    if os.path.isdir(path):
        path = os.path.join(path, "data.csv")
    with open(path, "w", newline="", encoding="utf-8") as csvfile:
        writer = csv.DictWriter(csvfile, fieldnames=columns)
        writer.writeheader()
        for row in data:
            writer.writerow(row)



import mariadb
from rich import print

from sensamag_utility.connection_manager import ConnectionManager
from sensamag_utility.csv_reader import write_data


def export_db(conn: mariadb.Connection, path: str):
    print(f"> Exporting data to: {path}")
    cur = conn.cursor(dictionary=True)
    try:
        cur.execute(
            """
        SELECT textreferences.Name AS Reference, textcontents.Text AS Content,
            localizationlanguages.Name AS Language, textcontents.Id AS ContentId,
            textcontents.TextReferences_Id AS ReferenceId, textcontents.Language_Id AS LanguageId
        FROM textcontents INNER JOIN (localizationlanguages, textreferences)
            ON textcontents.Language_Id = localizationlanguages.Id
            AND textcontents.TextReferences_Id = textreferences.Id
        """
        )
        headers = [row[0] for row in cur.description]
        write_data(path, cur, headers)
        print(
            f"> [bold green]Succesfully written [yellow]{cur.affected_rows}[/] rows to [yellow]{path}[/]!"
        )
    except mariadb.Error as exception:
        print(f"> [bold red]Error exporting data:[/] {exception}")



from rich import print
from rich.table import Table


def print_as_table(cur: dict, name: str = "Result"):
    table = Table(*[row[0] for row in cur.description], title=name)
    for row in cur:
        table.add_row(*[str(ele) for ele in row])
    print(table)



import mariadb
from rich import print

from sensamag_utility.utility import print_as_table


def list_languages(conn: mariadb.Connection):
    print("> Fetching languages table")
    cur = conn.cursor()
    try:
        cur.execute(
            """
        SELECT * FROM localizationlanguages
        """
        )
        print_as_table(cur, name="localizationlanguages")
        print(f"> [bold green]Fetched: {cur.affected_rows} rows")
    except mariadb.Error as exception:
        print(f"> [bold red]Error fetching languages:[/] {exception}")


def add_language(conn: mariadb.Connection, name: str, culture: str, priority: int):
    print(
        f"""
    > Adding new language:
    | > name: {name}
    | > culture: {culture}
    | > priority: {priority}
    """
    )
    cur = conn.cursor()
    try:
        cur.execute(
            "INSERT INTO localizationlanguages (Name,Culture, Priority) VALUES (?, ?, ?)",
            (name, culture, priority),
        )
        conn.commit()
        print(
            f"> [bold green]Language {name} is added successfully with {cur.lastrowid} id"
        )
    except mariadb.Error as exception:
        print(f"> [bold red]Error adding new language[/]: {exception}")


def remove_language(conn: mariadb.Connection, lang_id: int):
    print(f"> Removing language by id: {lang_id}")
    cur = conn.cursor()
    try:
        cur.execute("DELETE FROM localizationlanguages WHERE Id = (?)", (lang_id,))
        conn.commit()
        print(f"> [bold green]Language with id: {lang_id} is removed successfully!")
    except mariadb.Error as exception:
        print(f"> [bold red]Error removing language:[/] {exception}")
